#include "td_main.h"
#include <raymath.h>

Tower towers[TOWER_MAX_COUNT];
int towerCount = 0;

void TowerInit()
{
  for (int i = 0; i < TOWER_MAX_COUNT; i++)
  {
    towers[i] = (Tower){0};
  }
  towerCount = 0;
}

static void TowerGunUpdate(Tower *tower)
{
  if (tower->cooldown <= 0)
  {
    Enemy *enemy = EnemyGetClosestToCastle(tower->x, tower->y, 3.0f);
    if (enemy)
    {
      tower->cooldown = 0.5f;
      // shoot the enemy; determine future position of the enemy
      float bulletSpeed = 1.0f;
      float bulletDamage = 3.0f;
      Vector2 velocity = enemy->simVelocity;
      Vector2 futurePosition = EnemyGetPosition(enemy, gameTime.time - enemy->startMovingTime, &velocity, 0);
      Vector2 towerPosition = {tower->x, tower->y};
      float eta = Vector2Distance(towerPosition, futurePosition) / bulletSpeed;
      for (int i = 0; i < 8; i++) {
        velocity = enemy->simVelocity;
        futurePosition = EnemyGetPosition(enemy, gameTime.time - enemy->startMovingTime + eta, &velocity, 0);
        float distance = Vector2Distance(towerPosition, futurePosition);
        float eta2 = distance / bulletSpeed;
        if (fabs(eta - eta2) < 0.01f) {
          break;
        }
        eta = (eta2 + eta) * 0.5f;
      }
      ProjectileTryAdd(PROJECTILE_TYPE_BULLET, enemy, towerPosition, futurePosition, 
        bulletSpeed, bulletDamage);
      enemy->futureDamage += bulletDamage;
    }
  }
  else
  {
    tower->cooldown -= gameTime.deltaTime;
  }
}

Tower *TowerGetAt(int16_t x, int16_t y)
{
  for (int i = 0; i < towerCount; i++)
  {
    if (towers[i].x == x && towers[i].y == y)
    {
      return &towers[i];
    }
  }
  return 0;
}

Tower *TowerTryAdd(uint8_t towerType, int16_t x, int16_t y)
{
  if (towerCount >= TOWER_MAX_COUNT)
  {
    return 0;
  }

  Tower *tower = TowerGetAt(x, y);
  if (tower)
  {
    return 0;
  }

  tower = &towers[towerCount++];
  tower->x = x;
  tower->y = y;
  tower->towerType = towerType;
  tower->cooldown = 0.0f;
  tower->damage = 0.0f;
  return tower;
}

Tower *GetTowerByType(uint8_t towerType)
{
  for (int i = 0; i < towerCount; i++)
  {
    if (towers[i].towerType == towerType)
    {
      return &towers[i];
    }
  }
  return 0;
}

int GetTowerCosts(uint8_t towerType)
{
  switch (towerType)
  {
  case TOWER_TYPE_BASE:
    return 0;
  case TOWER_TYPE_GUN:
    return 6;
  case TOWER_TYPE_WALL:
    return 2;
  }
  return 0;
}

float TowerGetMaxHealth(Tower *tower)
{
  switch (tower->towerType)
  {
  case TOWER_TYPE_BASE:
    return 10.0f;
  case TOWER_TYPE_GUN:
    return 3.0f;
  case TOWER_TYPE_WALL:
    return 5.0f;
  }
  return 0.0f;
}

void TowerDraw()
{
  for (int i = 0; i < towerCount; i++)
  {
    Tower tower = towers[i];
    DrawCube((Vector3){tower.x, 0.125f, tower.y}, 1.0f, 0.25f, 1.0f, GRAY);
    switch (tower.towerType)
    {
    case TOWER_TYPE_BASE:
      DrawCube((Vector3){tower.x, 0.4f, tower.y}, 0.8f, 0.8f, 0.8f, MAROON);
      break;
    case TOWER_TYPE_GUN:
      DrawCube((Vector3){tower.x, 0.2f, tower.y}, 0.8f, 0.4f, 0.8f, DARKPURPLE);
      break;
    case TOWER_TYPE_WALL:
      DrawCube((Vector3){tower.x, 0.5f, tower.y}, 1.0f, 1.0f, 1.0f, LIGHTGRAY);
      break;
    }
  }
}

void TowerUpdate()
{
  for (int i = 0; i < towerCount; i++)
  {
    Tower *tower = &towers[i];
    switch (tower->towerType)
    {
    case TOWER_TYPE_GUN:
      TowerGunUpdate(tower);
      break;
    }
  }
}
